/** @jsxImportSource @emotion/react */
import { SerializedStyles } from "@emotion/react";
import * as React from "react";
import { ButtonSize } from "./Button";
export declare type InputSize = "sm" | "md" | "lg";
export interface InputGroupProps extends React.HTMLAttributes<HTMLDivElement> {
    id?: string;
    /** A label is required for accessibility purposes. Use `hideLabel` to hide it. */
    label: string;
    /** Visually hide the label. It remains accessible to screen readers. */
    hideLabel?: boolean;
    error?: string | React.ReactNode;
    /** Optional help text */
    helpText?: string;
    /** A single input element */
    children?: React.ReactNode;
    labelStyle?: SerializedStyles;
    labelTextStyle?: SerializedStyles;
}
/**
原来版本的输入分组项目；    新版本使用InputLine
 */
export declare const InputGroup: React.FunctionComponent<InputGroupProps>;
export interface InputBaseProps extends React.InputHTMLAttributes<HTMLInputElement> {
    /** The size of the input element */
    inputSize?: InputSize;
}
/**
 * Our basic Input element. Use this when building customized
 * forms. Otherwise, stick with InputGroup
 * 原先InputBase 改名成 InputRefBase; 比如ComboBox才会需要使用forwardRef形式的;
 */
export declare const InputRefBase: React.ForwardRefExoticComponent<InputBaseProps & React.RefAttributes<HTMLInputElement>>;
/**
 * 正常都不需要使用forwardRef形式的，可大大地加快速度。 InputRefBase是ref版本;
 * 性能优化，去掉了React.forwardRef 改成React.FunctionComponent
 * InputSimple普通FunctionComponent代替React.forwardRef(safeBind({ ref }就能从365ms变240ms啦。
 */
export declare const InputBase: React.FunctionComponent<InputBaseProps>;
export interface InputProps extends InputBaseProps {
    /** 控制是否满上宽度;
     *  需要在<input> 上 去控制大尺寸上限的width:  ，以及自适应屏幕大小后的 max-width: 缩小尺寸。
     * */
    fullWidth?: boolean;
}
/**
 * Input 比起InputBase，外面多包裹一个div以便于控制宽度和对齐。
 * style是父辈注入的样式: 父辈采用React.cloneElement(children, { style: {flex: '1 1 60%' } }注入样式。
 */
export declare const Input: React.FunctionComponent<InputProps>;
/**
 * InputRefComp 是ref传递版本的Input,特别情况用
 * InputRefComp 比起 InputRefBase多个一层<div>可以注入控制样式,外面多包裹一个div以便于控制宽度和对齐。。
 */
export declare const InputRefComp: React.ForwardRefExoticComponent<InputProps & React.RefAttributes<HTMLInputElement>>;
export interface TextAreaProps extends React.TextareaHTMLAttributes<HTMLTextAreaElement> {
    /** The size of the textarea element */
    inputSize?: InputSize;
}
/**
 * Textarea version of InputBase
 */
export declare const TextArea: React.FunctionComponent<TextAreaProps>;
export interface LabelProps extends React.LabelHTMLAttributes<HTMLLabelElement> {
    hide?: boolean;
    htmlFor: string;
    textStyle?: SerializedStyles;
}
/**
 * A styled Label to go along with input elements
 */
export declare const Label: React.FunctionComponent<LabelProps>;
export interface SelectProps extends React.SelectHTMLAttributes<HTMLSelectElement> {
    /** The size of the select box */
    inputSize?: InputSize;
    divStyle?: SerializedStyles;
}
/**
 * multiple在手机上可以不展开列表了，但是选择2个以上在编辑框上面只能显示数目，不会显示选择那些必须点击进入才能看见。在电脑上面还是列表全部展开的形式。
 * 若multiple=true 导致直接拉开了，样式窗口高度都改变，失去UI一致性了『感觉不友好啊』。
 * 光光靠w3c标准也行不通, 难以通用，必须自己定制。
 * ComboBox不允许multiple；但是Select不允许自主添加新的列表项目，Select允许列表附加label描述文字代替value;
 * 若触摸屏 不能支持multiple形态的的Select，只能单选。?浏览器版本升级，但是还会有返回的数据【】集合机制问题,value=[values1,2]。
 * multiple 只是摊开选择列表，onChange  .target.value无法提供多选数组，必须额外维护可以多选的被选中状态和数据数组。
 * 原来版本是用这样复制 children 的这个 <option></option> 也能够用...other直接复制<select 的底下标签去。
 * style?: React.CSSProperties;
 */
export declare const Select: React.FunctionComponent<SelectProps>;
export interface CheckProps extends React.InputHTMLAttributes<HTMLInputElement> {
    /** A label for the checkmark. */
    label: string;
}
export declare const Check: React.FunctionComponent<CheckProps>;
export interface SuffixInputProps extends InputBaseProps {
    textStyle?: SerializedStyles;
}
/** SuffixInput 带单位标注的输入框
 * A styled Label to go along with input elements
 * 若children有，就是有附带单位后缀串的模式；
 * 带单位后缀的说明，70%给输入框，后面30%给叙述单位字串{空格也算}
 * 输入的 单位说明 字符串 放在 <Text className="Suffix__text"  >
 * 没有ref注入的版本；
 */
export declare const SuffixInput: React.FunctionComponent<SuffixInputProps>;
export interface InputLineProps extends InputGroupProps {
    lineStyle?: SerializedStyles;
    fitable?: boolean;
}
/**
自适应屏幕flexBox布局：不要设置固定的width和min-width，可以设置max-width；根据屏幕宽策划1列2列还是更多列的并列，或是更高层次嵌套或隐藏或显示一小半边天区域。
不要对InputGroupLine的上一级div定义固定宽度，自适应和固定width: px只能二者选其一；宽度定了对小屏幕场景就有滚动条，而不是自适应缩小flexBox布局。
修改InputGroup排版模式; 并排模式，根据屏幕自适应。支持 2 个模式的布局安排结构。
 性能优化，旧版本InputGroupLine=680ms; 新的InputLine=600ms;
这两个布局占位参数 error, helpText感觉意义不大。
 InputGroupContext才是最关键的，子孙组件uid和LabelText标签挂接。

一般<section>出现在文档文章大纲中。一般通过是否包含一个标题 <h1>-<h6>作为子节点 来辨识<section>。
 InputLine只能支持下面有唯一的一个儿子组件的。 可以使用<div>包裹多个孙子组件。 举例如下：
     <div>
         <Check label="Male" checked />
         <Check label="Female" />
     </div>
 儿子若这样<div style={{border: `1px solid ${theme.colors.border.default}`}} > 实际style就将无法起作用；
 儿子像这样<div style="border:dotted 2px black; background-color:Yellow;height:100px;" 实际将无法起作用。 style={css`margin: 1rem 2rem;`} > style={css({ padding: "1rem 2rem" })} >将无法起作用;
 儿子像这样<div css={css({ padding: "1rem 2rem" })}，这样 css={css`margin: 1rem 2rem;`} 就会生效启用样式。
 儿子组件第一个div或tag若像这样 <div css={{marginTop: 19,backgroundColor: 'red'}} > 就会生效启用样式。
 儿子组件<div css={{flex: '1 1 20%',backgroundColor: 'red'}} 本组件从父辈修改注入style: {flex: '1 1 60%' }就会覆盖掉儿子div的flex: '1 1 20%'的原本样式。React.cloneElement(child模式 替换了。
总之: 儿子组件必须用css={}；第三方独立组件也许要在外边加一层次div来调整样式。
 InputLine是单个输入包裹组件，外部再多搞一层布局组件LineColumn来配合。
 */
export declare const InputLine: React.FunctionComponent<InputLineProps>;
export interface InputDatalistProps extends InputBaseProps {
    /** 控制是否满上宽度;
     *  需要在<input> 上 去控制大尺寸上限的width:  ，以及自适应屏幕大小后的 max-width: 缩小尺寸。
     * */
    fullWidth?: boolean;
    datalist?: any[];
    onListChange: (value: string) => void;
}
/**底层浏览器已经做了现成的支持ComboBox，何必自己再搞那么麻烦呢。
 * 直接用W3C浏览器提供的<datalist标签？list做关联id,来做组合输入框，代替ComboBox组件
 *<datalist id=""> 实际上可以脱离<input 而存在的，比如作为全局的不改动的<datalist集合放在一起。 管理？分离掉了；
 * 还是自己包含<datalist维护好了。
 * InputDatalist对比ComboBox组件的弱点1非空编辑不能点击就显示所有列表2手机上列表展开位置与窗口大小太矮了3<option的label与value都显示出来感觉不好。
 * ，但比ComboBox有个有优势：1能够记住当前用户输入用过的以后再用。
 * 不过一般输入完成后再做点击修改的概率也不大，手机版本显示区域太小问题以后浏览器版本可能改进的。
若组件附带参数这样的<InputDatalist css={{flex: '1 1 80%'}} > 将会导致...other实际接收这里的样式参数，样式不会作用于第一个div的。
若组件附带参数这样的<InputDatalist style={{flex: '1 1 80%',backgroundColor: "pink"}}> 实际第一个div或者...other位置不会获得任何样式，也就是说style={{是无效的。
 <InputDatalist style={{border: `1px solid ${theme.colors.border.default}`}} >  实际style就将无法起作用；
 <InputDatalist style="border:dotted 2px black; background-color:Yellow;height:100px;"  实际将无法起作用。
<InputDatalist 只能使用css来注入样式；<InputDatalist css={{ 方式注入的样式，只能在...other占位替换位置接收样式， 而不能用参数style参数css类似这样的声明参数方式引入的样式。
但是若父辈采用React.cloneElement( React.Children.only(children),{style: {flex: '1 1 60%' } }这样模式，就能够注入style，而且是style参数获得的声明参数方式引入样式。
 */
export declare const InputDatalist: React.FunctionComponent<InputDatalistProps>;
export interface CheckSwitchProps extends React.InputHTMLAttributes<HTMLInputElement> {
    /** The size of the Switch. 多大高度方向的尺寸 */
    hsize?: ButtonSize;
}
/**
 * 从‘react-switch’导入组件,做个可替代Check的更加美观的开关组件。
 * @param disabled
 * @param topDivStyle
 */
export declare const CheckSwitch: React.FunctionComponent<CheckSwitchProps>;
