import { __rest } from "tslib";
import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { css } from "@emotion/react";
import * as React from "react";
import { Text, LabelText } from "./Text";
import VisuallyHidden from "@reach/visually-hidden";
import PropTypes from "prop-types";
import { alpha } from "./Theme/colors";
import { useUid } from "./Hooks/use-uid";
import { useTheme } from "./Theme/Providers";
import { getHeight, focusShadow } from "./Button";
import { IconAlertCircle, IconChevronDown } from "./Icons";
import { safeBind } from "./Hooks/compose-bind";
import Switch from "react-switch";
/*
自适应布局，容器父组件不应当设置width固定的px，否则内部组件元素{已经为屏幕宽度自适应适配的组件}都会被动拉伸宽度，失去了效果。
*/
const getInputSizes = (theme) => ({
    sm: css({
        fontSize: theme.fontSizes[0],
        padding: "0.25rem 0.5rem"
    }),
    md: css({
        fontSize: theme.fontSizes[1],
        padding: "0.5rem 0.75rem"
    }),
    lg: css({
        fontSize: theme.fontSizes[2],
        padding: "0.65rem 1rem"
    })
});
const InputGroupContext = React.createContext({
    uid: undefined,
    error: undefined
});
/**
原来版本的输入分组项目；    新版本使用InputLine
 */
export const InputGroup = (_a) => {
    var { id, label, children, error, helpText, hideLabel, labelStyle, labelTextStyle } = _a, other = __rest(_a, ["id", "label", "children", "error", "helpText", "hideLabel", "labelStyle", "labelTextStyle"]);
    const uid = useUid(id);
    const theme = useTheme();
    const isDark = theme.colors.mode === "dark";
    const danger = isDark
        ? theme.colors.intent.danger.light
        : theme.colors.intent.danger.base;
    return (_jsxs("section", Object.assign({ className: "InputGroup", css: {
            marginTop: theme.spaces.md,
            "&.InputGroup:first-of-type": {
                marginTop: 0
            }
        } }, other, { children: [_jsx(Label, Object.assign({ hide: hideLabel, htmlFor: uid, css: labelStyle, textStyle: labelTextStyle }, { children: label }), void 0),
            _jsx(InputGroupContext.Provider, Object.assign({ value: {
                    uid,
                    error
                } }, { children: children }), void 0),
            error && typeof error === "string" ? (_jsxs("div", Object.assign({ className: "InputGroup__error", css: {
                    alignItems: "center",
                    marginTop: theme.spaces.sm,
                    display: "flex"
                } }, { children: [_jsx(IconAlertCircle, { size: "sm", color: danger }, void 0),
                    _jsx(Text, Object.assign({ css: {
                            display: "block",
                            marginLeft: theme.spaces.xs,
                            fontSize: theme.fontSizes[0],
                            color: danger
                        } }, { children: error }), void 0)] }), void 0)) : (error),
            helpText && (_jsx(Text, Object.assign({ className: "InputGroup__help", css: {
                    display: "block",
                    marginTop: theme.spaces.xs,
                    color: theme.colors.text.muted,
                    fontSize: theme.fontSizes[0]
                }, variant: "body" }, { children: helpText }), void 0))] }), void 0));
};
InputGroup.propTypes = {
    label: PropTypes.string.isRequired,
    hideLabel: PropTypes.bool,
    helpText: PropTypes.string,
    error: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
    id: PropTypes.string,
    children: PropTypes.node
};
function shadowBorder(color, opacity) {
    return `0 0 0 2px transparent inset, 0 0 0 1px ${alpha(color, opacity)} inset`;
}
function getBaseStyles(theme) {
    const dark = theme.colors.mode === "dark";
    const baseStyles = css({
        display: "block",
        width: "100%",
        lineHeight: theme.lineHeights.body,
        color: theme.colors.text.default,
        backgroundColor: "transparent",
        backgroundImage: "none",
        backgroundClip: "padding-box",
        WebkitFontSmoothing: "antialiased",
        WebkitTapHighlightColor: "transparent",
        WebkitAppearance: "none",
        boxSizing: "border-box",
        touchAction: "manipulation",
        fontFamily: theme.fonts.base,
        border: "none",
        boxShadow: dark
            ? shadowBorder(theme.colors.palette.gray.lightest, 0.14)
            : shadowBorder(theme.colors.palette.gray.dark, 0.2),
        borderRadius: theme.radii.sm,
        transition: "background 0.25s cubic-bezier(0.35,0,0.25,1), border-color 0.15s cubic-bezier(0.35,0,0.25,1), box-shadow 0.15s cubic-bezier(0.35,0,0.25,1)",
        "::placeholder": {
            color: alpha(theme.colors.text.default, 0.45)
        },
        ":focus": {
            boxShadow: dark
                ? focusShadow(alpha(theme.colors.palette.blue.light, 0.5), alpha(theme.colors.palette.gray.dark, 0.4), alpha(theme.colors.palette.gray.light, 0.2))
                : focusShadow(alpha(theme.colors.palette.blue.dark, 0.1), alpha(theme.colors.palette.gray.dark, 0.2), alpha(theme.colors.palette.gray.dark, 0.05)),
            outline: "none"
        },
        ":disabled": {
            opacity: dark ? 0.4 : 0.8,
            background: theme.colors.background.tint1,
            cursor: "not-allowed",
            boxShadow: dark
                ? shadowBorder(theme.colors.palette.gray.lightest, 0.15)
                : shadowBorder(theme.colors.palette.gray.dark, 0.12)
        },
        ":active": {
            background: theme.colors.background.tint1
        }
    });
    return baseStyles;
}
function useActiveStyle() {
    const [active, setActive] = React.useState(false);
    return {
        bind: {
            onTouchStart: () => setActive(true),
            onTouchEnd: () => setActive(false)
        },
        active
    };
}
function useSharedStyle() {
    const theme = useTheme();
    const errorStyles = {
        boxShadow: shadowBorder(theme.colors.intent.danger.base, 0.45)
    };
    const baseStyles = React.useMemo(() => getBaseStyles(theme), [theme]);
    const inputSizes = React.useMemo(() => getInputSizes(theme), [theme]);
    const activeBackground = css({ background: theme.colors.background.tint1 });
    return {
        baseStyles,
        inputSizes,
        activeBackground,
        errorStyles
    };
}
/**
 * Our basic Input element. Use this when building customized
 * forms. Otherwise, stick with InputGroup
 * 原先InputBase 改名成 InputRefBase; 比如ComboBox才会需要使用forwardRef形式的;
 */
export const InputRefBase = React.forwardRef((_a, ref) => {
    var { autoComplete, autoFocus, inputSize = "md" } = _a, other = __rest(_a, ["autoComplete", "autoFocus", "inputSize"]);
    const { uid, error } = React.useContext(InputGroupContext);
    const { bind, active } = useActiveStyle();
    const { baseStyles, inputSizes, activeBackground, errorStyles } = useSharedStyle();
    const height = getHeight(inputSize);
    return (_jsx("input", Object.assign({ id: uid, className: "Input", autoComplete: autoComplete, autoFocus: autoFocus }, bind, { css: [
            baseStyles,
            inputSizes[inputSize],
            active && activeBackground,
            error && errorStyles,
            { height }
        ] }, safeBind({ ref }, other)), void 0));
});
/**
 * 正常都不需要使用forwardRef形式的，可大大地加快速度。 InputRefBase是ref版本;
 * 性能优化，去掉了React.forwardRef 改成React.FunctionComponent
 * InputSimple普通FunctionComponent代替React.forwardRef(safeBind({ ref }就能从365ms变240ms啦。
 */
export const InputBase = (_a) => {
    var { autoComplete, autoFocus, inputSize = "md" } = _a, other = __rest(_a, ["autoComplete", "autoFocus", "inputSize"]);
    const { uid, error } = React.useContext(InputGroupContext);
    const { bind, active } = useActiveStyle();
    const { baseStyles, inputSizes, activeBackground, errorStyles } = useSharedStyle();
    const height = getHeight(inputSize);
    return (_jsx("input", Object.assign({ id: uid, className: "Input", autoComplete: autoComplete, autoFocus: autoFocus }, bind, { css: [
            baseStyles,
            inputSizes[inputSize],
            active && activeBackground,
            error && errorStyles,
            { height }
        ] }, other), void 0));
};
InputBase.propTypes = {
    inputSize: PropTypes.oneOf(["sm", "md", "lg"]),
    autoComplete: PropTypes.string,
    autoFocus: PropTypes.bool
};
/**
 * Input 比起InputBase，外面多包裹一个div以便于控制宽度和对齐。
 * style是父辈注入的样式: 父辈采用React.cloneElement(children, { style: {flex: '1 1 60%' } }注入样式。
 */
export const Input = (_a) => {
    var { autoComplete, autoFocus, inputSize = "md", fullWidth = true, style } = _a, other = __rest(_a, ["autoComplete", "autoFocus", "inputSize", "fullWidth", "style"]);
    const { uid, error } = React.useContext(InputGroupContext);
    const { bind, active } = useActiveStyle();
    const { baseStyles, inputSizes, activeBackground, errorStyles } = useSharedStyle();
    const height = getHeight(inputSize);
    return (_jsx("div", Object.assign({ css: Object.assign({ textAlign: 'left', width: "100%" }, style) }, { children: _jsx("input", Object.assign({ id: uid, className: "Input", autoComplete: autoComplete, autoFocus: autoFocus }, bind, { css: [
                baseStyles,
                inputSizes[inputSize],
                active && activeBackground,
                error && errorStyles,
                { height },
                !fullWidth && {
                    width: 'unset',
                }
            ] }, other), void 0) }), void 0));
};
/**
 * InputRefComp 是ref传递版本的Input,特别情况用
 * InputRefComp 比起 InputRefBase多个一层<div>可以注入控制样式,外面多包裹一个div以便于控制宽度和对齐。。
 */
export const InputRefComp = React.forwardRef((_a, ref) => {
    var { autoComplete, autoFocus, inputSize = "md", fullWidth = true, style } = _a, other = __rest(_a, ["autoComplete", "autoFocus", "inputSize", "fullWidth", "style"]);
    const { uid, error } = React.useContext(InputGroupContext);
    const { bind, active } = useActiveStyle();
    const { baseStyles, inputSizes, activeBackground, errorStyles } = useSharedStyle();
    const height = getHeight(inputSize);
    return (_jsx("div", Object.assign({ css: Object.assign({ textAlign: 'left', width: "100%" }, style) }, { children: _jsx("input", Object.assign({ id: uid, className: "Input", autoComplete: autoComplete, autoFocus: autoFocus }, bind, { css: [
                baseStyles,
                inputSizes[inputSize],
                active && activeBackground,
                error && errorStyles,
                { height },
                !fullWidth && {
                    width: 'unset',
                }
            ] }, safeBind({ ref }, other)), void 0) }), void 0));
});
/**
 * Textarea version of InputBase
 */
export const TextArea = (_a) => {
    var { inputSize = "md", style } = _a, other = __rest(_a, ["inputSize", "style"]);
    const { bind, active } = useActiveStyle();
    const { baseStyles, inputSizes, activeBackground, errorStyles } = useSharedStyle();
    const { uid, error } = React.useContext(InputGroupContext);
    return (_jsx("textarea", Object.assign({ className: "TextArea", id: uid }, bind, { css: [
            baseStyles,
            inputSizes[inputSize],
            {
                overflow: "auto",
                resize: "vertical",
            },
            active && activeBackground,
            error && errorStyles,
            Object.assign({}, style)
        ] }, other), void 0));
};
TextArea.propTypes = {
    inputSize: PropTypes.oneOf(["sm", "md", "lg"])
};
/**
 * A styled Label to go along with input elements
 */
export const Label = (_a) => {
    var { children, hide, textStyle } = _a, other = __rest(_a, ["children", "hide", "textStyle"]);
    const theme = useTheme();
    const child = (_jsx("label", Object.assign({ className: "Label", css: {
            display: "inline-block",
            marginBottom: hide ? 0 : theme.spaces.sm
        } }, other, { children: _jsx(Text, Object.assign({ className: "Label__text", variant: "subtitle", css: textStyle }, { children: children }), void 0) }), void 0));
    return hide ? _jsx(VisuallyHidden, { children: child }, void 0) : child;
};
Label.propTypes = {
    hide: PropTypes.bool,
    children: PropTypes.node
};
/**
 * multiple在手机上可以不展开列表了，但是选择2个以上在编辑框上面只能显示数目，不会显示选择那些必须点击进入才能看见。在电脑上面还是列表全部展开的形式。
 * 若multiple=true 导致直接拉开了，样式窗口高度都改变，失去UI一致性了『感觉不友好啊』。
 * 光光靠w3c标准也行不通, 难以通用，必须自己定制。
 * ComboBox不允许multiple；但是Select不允许自主添加新的列表项目，Select允许列表附加label描述文字代替value;
 * 若触摸屏 不能支持multiple形态的的Select，只能单选。?浏览器版本升级，但是还会有返回的数据【】集合机制问题,value=[values1,2]。
 * multiple 只是摊开选择列表，onChange  .target.value无法提供多选数组，必须额外维护可以多选的被选中状态和数据数组。
 * 原来版本是用这样复制 children 的这个 <option></option> 也能够用...other直接复制<select 的底下标签去。
 * style?: React.CSSProperties;
 */
export const Select = (_a) => {
    var { multiple, inputSize = "md", divStyle, style } = _a, other = __rest(_a, ["multiple", "inputSize", "divStyle", "style"]);
    const theme = useTheme();
    const inputSizes = getInputSizes(theme);
    const { uid, error } = React.useContext(InputGroupContext);
    const selectSize = {
        sm: inputSizes.sm,
        md: inputSizes.md,
        lg: inputSizes.lg
    };
    const dark = theme.colors.mode === "dark";
    const height = getHeight(inputSize);
    //因需要Select组件的max-width；导致Select若放InputGroupLine下在宽松模式一行内显示Label和Select的场景，在Select组件头层div设置宽度将会使得flex无法对齐两个项目；所以再套入一个div。
    //像这样<div style="border:dotted 2px black; background-color:Yellow;height:100px;" 是html底层样式表达法,实际无法实现注入的。
    //而styles=是变量名字。  style={{ padding: "0.43rem", textAlign: 'center' }}这是对象写法。
    //使用<div style={ style } 注入的将会是独立的style样式；调试开闭是只能针对单一个元素开关。
    //使用<div css={[style as any  注入的将会是class样式模组。调试开闭是可以开关多个相同class的元素。 两个模式都能有效果 style css。
    return (_jsx("div", Object.assign({ css: Object.assign({ textAlign: 'left' }, style) }, { children: _jsxs("div", Object.assign({ className: "Select", css: [
                {
                    //这个position: "relative"因为其下级的position: "absolute"定位的小小图标所约束。所以只能relative或sticky其他position取值不行。
                    position: "relative",
                },
                divStyle,
            ] }, { children: [_jsx("select", Object.assign({ className: "Select__input", id: uid, css: [
                        selectSize[inputSize],
                        {
                            WebkitAppearance: "none",
                            display: "block",
                            width: "100%",
                            // lineHeight: theme.lineHeights.body,
                            height: multiple ? 'unset' : height,
                            color: theme.colors.text.default,
                            background: "transparent",
                            fontFamily: theme.fonts.base,
                            boxShadow: `0 0 0 2px transparent inset, 0 0 0 1px ${dark
                                ? alpha(theme.colors.palette.gray.lightest, 0.14)
                                : alpha(theme.colors.palette.gray.dark, 0.2)} inset`,
                            border: "none",
                            backgroundClip: "padding-box",
                            borderRadius: theme.radii.sm,
                            margin: 0,
                            ":disabled": {
                                ":disabled": {
                                    opacity: dark ? 0.4 : 0.8,
                                    background: theme.colors.background.tint1,
                                    cursor: "not-allowed",
                                    boxShadow: `0 0 0 2px transparent inset, 0 0 0 1px ${dark
                                        ? alpha(theme.colors.palette.gray.lightest, 0.15)
                                        : alpha(theme.colors.palette.gray.dark, 0.12)} inset`
                                }
                            },
                            ":focus": {
                                borderColor: theme.colors.palette.blue.base,
                                boxShadow: dark
                                    ? focusShadow(alpha(theme.colors.palette.blue.light, 0.5), alpha(theme.colors.palette.gray.dark, 0.4), alpha(theme.colors.palette.gray.light, 0.2))
                                    : focusShadow(alpha(theme.colors.palette.blue.dark, 0.1), alpha(theme.colors.palette.gray.dark, 0.2), alpha(theme.colors.palette.gray.dark, 0.05)),
                                outline: 0
                            }
                        },
                        error && {
                            boxShadow: shadowBorder(theme.colors.intent.danger.base, 0.45)
                        }
                    ], multiple: multiple }, other), void 0),
                !multiple && (_jsx(IconChevronDown, { className: "Select__icon", color: theme.colors.text.muted, css: {
                        position: "absolute",
                        top: "50%",
                        right: "0.75rem",
                        transform: "translateY(-50%)",
                        pointerEvents: "none"
                    } }, void 0))] }), void 0) }), void 0));
};
Select.propTypes = {
    inputSize: PropTypes.oneOf(["sm", "md", "lg"]),
    multiple: PropTypes.bool
};
export const Check = (_a) => {
    var { label, id, disabled, style } = _a, other = __rest(_a, ["label", "id", "disabled", "style"]);
    const uid = useUid(id);
    const theme = useTheme();
    return (_jsxs("div", Object.assign({ className: "Check", css: [
            Object.assign({ display: "flex", alignItems: "center" }, style)
        ] }, { children: [_jsx("input", Object.assign({ disabled: disabled, className: "Check__input", type: "checkbox", id: uid, css: [
                    {
                        height: '2rem',
                        width: '2rem',
                        display: 'inline-flex',
                        paddingLeft: '1rem',
                    },
                ] }, other), void 0),
            _jsx("label", Object.assign({ className: "Check__label", css: {
                    opacity: disabled ? 0.6 : undefined,
                    marginLeft: theme.spaces.xs
                }, htmlFor: uid }, { children: _jsx(Text, { children: label }, void 0) }), void 0)] }), void 0));
};
Check.propTypes = {
    label: PropTypes.string.isRequired,
    id: PropTypes.string,
    disabled: PropTypes.bool
};
/** SuffixInput 带单位标注的输入框
 * A styled Label to go along with input elements
 * 若children有，就是有附带单位后缀串的模式；
 * 带单位后缀的说明，70%给输入框，后面30%给叙述单位字串{空格也算}
 * 输入的 单位说明 字符串 放在 <Text className="Suffix__text"  >
 * 没有ref注入的版本；
 */
export const SuffixInput = (_a) => {
    var { children, textStyle, inputSize, style } = _a, other = __rest(_a, ["children", "textStyle", "inputSize", "style"]);
    //const theme = useTheme();
    //children可以是非字符串的, 按钮等。
    return (_jsxs("div", Object.assign({ css: Object.assign({ textAlign: 'left' }, style) }, { children: [_jsx(InputBase, Object.assign({ inputSize: inputSize, css: {
                    display: "inline-block",
                    width: children ? "70%" : '100%',
                } }, other), void 0),
            typeof children === "string" ? (_jsx(Text, Object.assign({ className: "Suffix__text", variant: "subtitle", css: [
                    {
                        display: children ? "inline-flex" : 'none',
                        paddingLeft: '0.2rem'
                    },
                    textStyle
                ] }, { children: children }), void 0)) : (children)] }), void 0));
};
SuffixInput.propTypes = {
    children: PropTypes.node
};
/**
自适应屏幕flexBox布局：不要设置固定的width和min-width，可以设置max-width；根据屏幕宽策划1列2列还是更多列的并列，或是更高层次嵌套或隐藏或显示一小半边天区域。
不要对InputGroupLine的上一级div定义固定宽度，自适应和固定width: px只能二者选其一；宽度定了对小屏幕场景就有滚动条，而不是自适应缩小flexBox布局。
修改InputGroup排版模式; 并排模式，根据屏幕自适应。支持 2 个模式的布局安排结构。
 性能优化，旧版本InputGroupLine=680ms; 新的InputLine=600ms;
这两个布局占位参数 error, helpText感觉意义不大。
 InputGroupContext才是最关键的，子孙组件uid和LabelText标签挂接。

一般<section>出现在文档文章大纲中。一般通过是否包含一个标题 <h1>-<h6>作为子节点 来辨识<section>。
 InputLine只能支持下面有唯一的一个儿子组件的。 可以使用<div>包裹多个孙子组件。 举例如下：
     <div>
         <Check label="Male" checked />
         <Check label="Female" />
     </div>
 儿子若这样<div style={{border: `1px solid ${theme.colors.border.default}`}} > 实际style就将无法起作用；
 儿子像这样<div style="border:dotted 2px black; background-color:Yellow;height:100px;" 实际将无法起作用。 style={css`margin: 1rem 2rem;`} > style={css({ padding: "1rem 2rem" })} >将无法起作用;
 儿子像这样<div css={css({ padding: "1rem 2rem" })}，这样 css={css`margin: 1rem 2rem;`} 就会生效启用样式。
 儿子组件第一个div或tag若像这样 <div css={{marginTop: 19,backgroundColor: 'red'}} > 就会生效启用样式。
 儿子组件<div css={{flex: '1 1 20%',backgroundColor: 'red'}} 本组件从父辈修改注入style: {flex: '1 1 60%' }就会覆盖掉儿子div的flex: '1 1 20%'的原本样式。React.cloneElement(child模式 替换了。
总之: 儿子组件必须用css={}；第三方独立组件也许要在外边加一层次div来调整样式。
 InputLine是单个输入包裹组件，外部再多搞一层布局组件LineColumn来配合。
 */
export const InputLine = (_a) => {
    var { id, label, children, error, helpText, hideLabel, labelTextStyle, lineStyle, fitable = true } = _a, other = __rest(_a, ["id", "label", "children", "error", "helpText", "hideLabel", "labelTextStyle", "lineStyle", "fitable"]);
    const uid = useUid(id);
    const theme = useTheme();
    const isDark = theme.colors.mode === "dark";
    const danger = isDark
        ? theme.colors.intent.danger.light
        : theme.colors.intent.danger.base;
    //InputGroupLine包裹的下层的顶级组件的样式修改：下层顶级元素的display: block还算兼容可用; 但width: 100%影响较大。
    //topDivStyle方式： 会不被识别认得，底下的<div>不懂得该如何处理。
    //假如底下是div 加上 element.style {   flex: 1 1 60%;  }；
    const childNodeVar = (_jsx(InputGroupContext.Provider, Object.assign({ value: {
            uid,
            error
        } }, { children: 
        //只能支持一个儿子的。 可以为底下的<div>自动添加样式。
        //style: { flex: '1 1 60%' },      左边的项目文字描述　40%　右边输入框(含单位字符)占用60%
        React.cloneElement(React.Children.only(children), {
            ///topDivStyle: { flex: '1 1 60%' },
            style: { flex: '1 1 60%' }
        }) }), void 0));
    //这里htmlFor={uid}，标签label 和 input很可能分别属于不同div底下的。
    //const titleVar = (        );
    const TitleVar = _jsx(LabelText, Object.assign({ className: "Label__text", htmlFor: uid, css: [
            {
                //display: "inline-flex",
                textAlign: fitable ? "right" : "left",
                flex: '1 1 40%',
                paddingRight: '0.8rem',
                marginBottom: hideLabel ? 0 : theme.spaces.sm,
                wordBreak: 'break-word'
            },
            labelTextStyle
        ] }, { children: label }), void 0);
    //子孙的宽度没有固定px的，布局组件的子孙都是可在宽度上做自适应的。只有目标多少列排列是敲定固定的，也就是预期最大父窗口宽度场合可以安排最多几个列的元素。
    //布局子孙都是平等的，宽度都平均分配，预期高度在同一行排列也是均衡整齐或高度一致的，
    //输入Line组件的断线折腰宽度在布局组件上就的设置switchPx参数。
    //这外部还得搞个布局组件嵌套，布局组件来传递进来布局紧凑与否参数fitable。也就是遇到最小最小的父窗口宽度情形，在只安排单列元素场合下的，给输入Line组件紧凑提示。
    // checkParent(React.Children.only(children) as any);
    //上级组件注入的style样式，这里实际在 ...other所接纳了。 上一级控制本组件的第一层div或tag, 其它层次其它参数来控制样式。
    return (_jsxs("div", Object.assign({ className: "InputLine", css: {
            marginTop: theme.spaces.md,
            "&.InputLine:first-of-type": {
                marginTop: 0
            },
            textAlign: 'center'
        } }, other, { children: [_jsxs("div", Object.assign({ css: [
                    {
                        alignItems: "center",
                        justifyContent: "space-around",
                        display: "flex",
                        // flexWrap: 'wrap',
                        maxWidth: '950px',
                        margin: '0 auto',
                        paddingRight: 'unset',
                    },
                    lineStyle
                ] }, { children: [hideLabel ? _jsx(VisuallyHidden, { children: TitleVar }, void 0)
                        : TitleVar, fitable && childNodeVar] }), void 0), !fitable && childNodeVar, error && typeof error === "string" ? (_jsxs("div", Object.assign({ className: "InputGroup__error", css: {
                    alignItems: "center",
                    marginTop: theme.spaces.sm,
                    display: "flex",
                    justifyContent: 'center'
                } }, { children: [_jsx(IconAlertCircle, { size: "sm", color: danger }, void 0),
                    _jsx(Text, Object.assign({ css: {
                            display: "block",
                            marginLeft: theme.spaces.xs,
                            fontSize: theme.fontSizes[0],
                            color: danger
                        } }, { children: error }), void 0)] }), void 0)) : (error),
            helpText && (_jsx(Text, Object.assign({ className: "InputGroup__help", css: {
                    display: "inline-flex",
                    marginTop: theme.spaces.xs,
                    color: theme.colors.text.muted,
                    fontSize: theme.fontSizes[0]
                }, variant: "body" }, { children: helpText }), void 0))] }), void 0));
};
InputLine.propTypes = {
    label: PropTypes.string.isRequired,
    hideLabel: PropTypes.bool,
    helpText: PropTypes.string,
    error: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
    id: PropTypes.string,
    children: PropTypes.node
};
/**底层浏览器已经做了现成的支持ComboBox，何必自己再搞那么麻烦呢。
 * 直接用W3C浏览器提供的<datalist标签？list做关联id,来做组合输入框，代替ComboBox组件
 *<datalist id=""> 实际上可以脱离<input 而存在的，比如作为全局的不改动的<datalist集合放在一起。 管理？分离掉了；
 * 还是自己包含<datalist维护好了。
 * InputDatalist对比ComboBox组件的弱点1非空编辑不能点击就显示所有列表2手机上列表展开位置与窗口大小太矮了3<option的label与value都显示出来感觉不好。
 * ，但比ComboBox有个有优势：1能够记住当前用户输入用过的以后再用。
 * 不过一般输入完成后再做点击修改的概率也不大，手机版本显示区域太小问题以后浏览器版本可能改进的。
若组件附带参数这样的<InputDatalist css={{flex: '1 1 80%'}} > 将会导致...other实际接收这里的样式参数，样式不会作用于第一个div的。
若组件附带参数这样的<InputDatalist style={{flex: '1 1 80%',backgroundColor: "pink"}}> 实际第一个div或者...other位置不会获得任何样式，也就是说style={{是无效的。
 <InputDatalist style={{border: `1px solid ${theme.colors.border.default}`}} >  实际style就将无法起作用；
 <InputDatalist style="border:dotted 2px black; background-color:Yellow;height:100px;"  实际将无法起作用。
<InputDatalist 只能使用css来注入样式；<InputDatalist css={{ 方式注入的样式，只能在...other占位替换位置接收样式， 而不能用参数style参数css类似这样的声明参数方式引入的样式。
但是若父辈采用React.cloneElement( React.Children.only(children),{style: {flex: '1 1 60%' } }这样模式，就能够注入style，而且是style参数获得的声明参数方式引入样式。
 */
export const InputDatalist = (_a) => {
    var { autoComplete, autoFocus, inputSize = "md", fullWidth = true, datalist = [], style, onListChange } = _a, other = __rest(_a, ["autoComplete", "autoFocus", "inputSize", "fullWidth", "datalist", "style", "onListChange"]);
    const { uid, error } = React.useContext(InputGroupContext);
    const { bind, active } = useActiveStyle();
    const { baseStyles, inputSizes, activeBackground, errorStyles } = useSharedStyle();
    const height = getHeight(inputSize);
    //这个版本：不需要React.forwardRef(()=>{})的，注入ref性能损失， {...safeBind({ ref }, other)}
    //这底下的 css={{  ...style ,位置必须放在最后一个， 否则...style可能被顺序排在后面的同样名字参数所覆盖。
    //但是无法修改弹出的列表的样式。 <datalist 无法设置样式，浏览器自动设置的样式。
    //死活都改不了； 弹出列表datalist无法自己定做样式。
    return (_jsxs("div", Object.assign({ css: Object.assign({ textAlign: 'left', width: "100%" }, style) }, { children: [_jsx("datalist", Object.assign({ id: `list${uid}` }, { children: datalist.map((one, i) => {
                    return _jsx("option", { value: one }, i);
                }) }), void 0),
            _jsx("input", Object.assign({ id: uid, className: "Input", autoComplete: autoComplete, autoFocus: autoFocus }, bind, { css: [
                    baseStyles,
                    inputSizes[inputSize],
                    active && activeBackground,
                    error && errorStyles,
                    { height },
                    !fullWidth && {
                        width: 'unset',
                    }
                ] }, other, { list: `list${uid}`, onChange: e => onListChange(e.currentTarget.value || undefined) }), void 0)] }), void 0));
};
const getSwitchHeight = (size) => {
    if (size === "xs")
        return 16;
    else if (size === "sm")
        return 22;
    else if (size === "lg")
        return 36;
    else if (size === "xl")
        return 44;
    else
        return 28;
};
/**
 * 从‘react-switch’导入组件,做个可替代Check的更加美观的开关组件。
 * @param disabled
 * @param topDivStyle
 */
export const CheckSwitch = (_a) => {
    var { id, disabled, style, checked, onChange, hsize = "md" } = _a, other = __rest(_a, ["id", "disabled", "style", "checked", "onChange", "hsize"]);
    const { uid } = React.useContext(InputGroupContext);
    //不能用这个React.useCallback((checked,event,id) => {， 状态无法切换, 外面传递进来的onChange()还是旧的数据，无法更新成新数值。
    //onHandleChange用了 useCallback（,[]）: 就是捕获函数， onChange就无法变化了,所以被锁住更新。
    const onHandleChange = (checked) => {
        onChange(checked);
    };
    return (_jsx("div", Object.assign({ className: "Switch", css: Object.assign({ 
            //display: "flex",  若加上这"flex",导致textAlign会失去作用了。
            alignItems: "center", textAlign: 'left' }, style) }, other, { children: _jsx(Switch, { id: uid, checked: checked, onChange: onHandleChange, disabled: disabled, height: getSwitchHeight(hsize), width: getSwitchHeight(hsize) * 2 }, void 0) }), void 0));
};
CheckSwitch.propTypes = {
    id: PropTypes.string,
    disabled: PropTypes.bool,
    hsize: PropTypes.oneOf(["xs", "sm", "md", "lg", "xl"])
};
//测试
function checkParent(parent) {
    if (!parent) {
        console.error(new Error('empty parent element'));
        return;
    }
    switch (parent.tagName) {
        case 'area':
        case 'base':
        case 'br':
        case 'col':
        case 'embed':
        case 'hr':
        case 'img':
        case 'input':
        case 'keygen':
        case 'link':
        case 'menuitem':
        case 'meta':
        case 'param':
        case 'source':
        case 'track':
        case 'wbr':
        case 'script':
        case 'style':
        case 'textarea':
        case 'title':
            console.error(new Error('Unsupported parent tag name ' +
                parent.tagName.toLowerCase() +
                '.' +
                parent.className.replace(/\s+/, '.') +
                ' . Change the tag or wrap it in a supported tag(e.g. div).'));
    }
    const parentStyles = window.getComputedStyle(parent);
    if (parentStyles && parentStyles.getPropertyValue('position') === 'static') {
        console.warn(new Error('LineColumn: ' +
            "The 'position' CSS property of element " +
            parent.tagName.toLowerCase() +
            '.' +
            parent.className.replace(/\s+/, '.') +
            " should not be 'static'."));
    }
}
//# sourceMappingURL=Form.js.map