import { __rest } from "tslib";
import * as React from "react";
import defaultTheme from ".";
/**
 * API:
 *
 * You can create custom themes using Object.assign:
 *
 *
 * Pass that theme to the theme provider. A theme should always
 * provide a dark and light color mode.
 *
 * <ThemeProvider theme={customTheme}><App /></ThemeProvider>
 *
 *
 * When you want to use a dark mode:
 *
 * function App () {
 *  return <Dark><SomeContent /></Dark>
 * }
 *
 *
 * To consume a theme, use `useTheme` hook
 *
 * const theme = useTheme()
 * // theme.colors.text.default (will be dark or light depending on the mode)
 * // theme.colors.mode === 'dark' or 'light'
 */
const ThemeContext = React.createContext(defaultTheme);
export const ThemeProvider = ({ theme = defaultTheme, children }) => {
    return (React.createElement(ThemeContext.Provider, { value: theme }, children));
};
/**
 * A hook for consuming the theme
 */
export function useTheme() {
    return React.useContext(ThemeContext);
}
/**
 * 去掉forwardRef，改成普通函数组件。
 */
const ColorMode = (_a) => {
    var { colors, children } = _a, other = __rest(_a, ["colors", "children"]);
    const theme = useTheme();
    //依照colors参数调整Theme结果； memo is necessary to prevent unnecessary rerenders
    // https://reactjs.org/docs/context.html#caveats
    const adjustedTheme = React.useMemo(() => mergeColors(theme, colors), [
        theme,
        colors
    ]);
    //其实这里向.Children.only(children)传递的 ref 实际没有用处；
    //本组件ThemeContext才是核心，把组件包裹下的子孙组件提供一致的 colors Theme;
    //这个版本调用 ? children(adjustedTheme) 就会报错？ 仅仅是编译器报错?
    //只能在forwardRef版本中才能使用? children(adjustedTheme)这样的 函数式儿子。
    //本组件底下包裹的子组件是 函数式儿子举例说明  {(theme: Theme) => (  <div  theme.colors >  )}  这样子的，非正常儿孙组件。
    return (React.createElement(ThemeContext.Provider, { value: adjustedTheme }, typeof children === "function"
        ? children(adjustedTheme)
        : React.cloneElement(React.Children.only(children), Object.assign({}, other))));
};
/**
 * 旧版传递 ref , 这里Ref只是接续传递，非自身必须要的；
 */
const ColorRefMode = React.forwardRef((_a, ref) => {
    var { colors, children } = _a, other = __rest(_a, ["colors", "children"]);
    const theme = useTheme();
    // memo is necessary to prevent unnecessary rerenders
    // https://reactjs.org/docs/context.html#caveats
    const adjustedTheme = React.useMemo(() => mergeColors(theme, colors), [
        theme,
        colors
    ]);
    //这个forwardRef版本才能调用 ? children(adjustedTheme)   函数式儿子；
    //本组件底下包裹的子组件是 函数式儿子举例说明  {(theme: Theme) => (  <div  theme.colors >  )}  这样子的，非正常儿孙组件。
    //非Ref版本,只能添加类型转换 ? (children as RenderCallbackType)(adjustedTheme) ;仅仅是编译器报错?
    return (React.createElement(ThemeContext.Provider, { value: adjustedTheme }, typeof children === "function"
        ? children(adjustedTheme)
        : React.cloneElement(React.Children.only(children), Object.assign({ ref }, other))));
});
ColorMode.displayName = "ColorMode";
function mergeColors(theme, colors) {
    return Object.assign(Object.assign({}, theme), { shadows: colors.shadows, colors });
}
export const LightMode = (_a) => {
    var { children } = _a, other = __rest(_a, ["children"]);
    const theme = useTheme();
    return (React.createElement(ColorMode, Object.assign({ colors: theme.modes.light }, other), children));
};
/**
 * 旧版传递 ref, 这里Ref只是接续传递，非自身必须要的；
 */
export const LightRefMode = React.forwardRef((_a, ref) => {
    var { children } = _a, other = __rest(_a, ["children"]);
    const theme = useTheme();
    return (React.createElement(ColorRefMode, Object.assign({ colors: theme.modes.light, ref: ref }, other), children));
});
LightMode.displayName = "LightMode";
/**
 * Provide a dark theme
 */
export const DarkMode = (_a) => {
    var { children } = _a, other = __rest(_a, ["children"]);
    const theme = useTheme();
    return (React.createElement(ColorMode, Object.assign({ colors: theme.modes.dark }, other), children));
};
/**
 * 旧版传递 ref, 这里Ref只是接续传递，非自身必须要的；
 */
export const DarkRefMode = React.forwardRef((_a, ref) => {
    var { children } = _a, other = __rest(_a, ["children"]);
    const theme = useTheme();
    return (React.createElement(ColorRefMode, Object.assign({ colors: theme.modes.dark, ref: ref }, other), children));
});
DarkMode.displayName = "DarkMode";
//# sourceMappingURL=Providers.js.map